<?php
declare(strict_types=1);

namespace Inti\Media\Container;

use Interop\Container\ContainerInterface;
use Inti\Media\Handler;
use Inti\Media\Middleware\HttpMethodMiddlewareDecorator;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Laminas\Stratigility\MiddlewarePipe;
use Mezzio\MiddlewareFactory;
use Psr\Http\Server\MiddlewareInterface;

/**
 * Class ApplicationFactory
 *
 * @package Inti\Media\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-media
 */
final class ApplicationFactory implements FactoryInterface
{
    /**
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param array|null $options
     * @return MiddlewareInterface
     * @noinspection PhpMissingParamTypeInspection
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null): MiddlewareInterface
    {
        /** @var MiddlewareFactory $factory */
        $factory = $container->get(MiddlewareFactory::class);

        $app = new MiddlewarePipe();
        $app->pipe(new HttpMethodMiddlewareDecorator(['GET', 'HEAD'], $factory->lazy(Handler\Serve::class)));
        $app->pipe(new HttpMethodMiddlewareDecorator(['PUT', 'POST'], $factory->lazy(Handler\Upload::class)));

        return $app;
    }
}
