<?php
declare(strict_types=1);

namespace Inti\Media\Model;

use function intval;
use function ltrim;
use function preg_match;
use function preg_replace;
use function rawurldecode;
use function sprintf;
use function strtolower;

final class ImagePath extends ImageSize
{
    protected string $path;

    public function __construct(string $path)
    {
        parent::__construct();
        $path = ltrim($path, '/');

        if (preg_match('#^((thumb/)?(\d*)x(\d*)/)?(.+)$#', $path, $matches)) {
            $this->cropped = $matches[2] ? true : false;
            $this->width = $matches[3] ? intval($matches[3]) : null;
            $this->height = $matches[4] ? intval($matches[4]) : null;
            $this->path = $matches[5];
        }
    }

    public function getRawPath(): string
    {
        return $this->path;
    }

    public function getFullPath(): string
    {
        return sprintf(
            '%s%s%s',
            $this->cropped ? 'thumb/' : '',
            $this->isResized() ? sprintf('%dx%d/', $this->width, $this->height) : '',
            $this->path
        );
    }

    public function isResized(): bool
    {
        return isset($this->width) || isset($this->height);
    }

    public function sanitize(): self
    {
        $path = strtolower(rawurldecode($this->path));
        $this->path = (string) preg_replace('/[\s[\],]+/', '-', $path);

        return $this;
    }

    public function setExtension(string $extension): self
    {
        if (preg_match('/(.+)\.(.+)$/', $this->path, $matches)) {
            if ($matches[2] !== $extension) {
                $this->path = $matches[1] . '.' . $extension;
            }
        }

        return $this;
    }
}
