<?php
declare(strict_types=1);

namespace IntiTest\Media;

use Inti\Media\ConfigProvider;
use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\Diactoros\ServerRequestFactory;
use Laminas\HttpHandlerRunner\Emitter\SapiEmitter;
use Laminas\HttpHandlerRunner\RequestHandlerRunner;
use Laminas\ServiceManager\ServiceManager;
use Laminas\Stratigility\Middleware\ErrorResponseGenerator;
use Laminas\Stratigility\Middleware\PathMiddlewareDecorator;
use Laminas\Stratigility\MiddlewarePipe;

require dirname(__DIR__) . '/vendor/autoload.php';

ini_set('display_errors', '1');
error_reporting(E_ALL);

(function() {
    $config = (new ConfigAggregator([
        \Mezzio\ConfigProvider::class,
        \Inti\Filesystem\ConfigProvider::class,
        ConfigProvider::class,
        new ArrayProvider([
            'filesystems' => [
                'image' => [
                    'adapter' => 'image',
                    'options' => [
                        'location' => __DIR__ . '/_media',
                    ],
                ],
            ],
        ]),
    ]))->getMergedConfig();

    $config['dependencies']['services']['config'] = $config;
    $container = new ServiceManager($config['dependencies']);

    $host = new MiddlewarePipe();

    $app = $container->get('IntiMedia');
    $host->pipe(new PathMiddlewareDecorator('/media', $app));

    $server = new RequestHandlerRunner(
        $host,
        new SapiEmitter(),
        [ServerRequestFactory::class, 'fromGlobals'],
        new ErrorResponseGenerator()
    );

    $server->run();
})();
